"""
palaver definition: an unnecessarily elaborate or complex procedure
(A.K.A. CASA in a nutshell - fight me)
"""
import argparse
import logging
import pathlib
import sys
import os
from pycapo import CapoConfig

from casa_envoy.launchers import CalibrationLauncher, ImagingLauncher

"""
Launch CASA processing jobs via Workspaces
"""

logger = logging.getLogger("casa_envoy")
logger.setLevel(logging.INFO)
logger.addHandler(logging.StreamHandler(sys.stdout))


def _get_settings(cwd: pathlib.Path, args: list):
    use_casa = CapoConfig().getboolean("edu.nrao.archive.workspaces.ProcessingSettings.useCasa")
    casa_home = (
        CapoConfig().settings("edu.nrao.archive.workflow.config.CasaVersions").homeForReprocessing
    )

    parent_path = cwd
    root_dir = str(cwd.parent)
    processing_dir = str(cwd.stem)

    metadata = args[0]
    ppr = args[1]

    return {
        "useCasa": use_casa,
        "homeForReprocessing": casa_home,
        "rootDirectory": root_dir,
        "processingDirectory": processing_dir,
        "parent_path": parent_path,
        "metadata": metadata,
        "ppr": ppr,
    }


def arg_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        description="Workspaces CASA processing launcher",
        formatter_class=argparse.RawTextHelpFormatter,
    )
    parser.add_argument(
        "-c",
        "--standard-cal",
        nargs=2,
        action="store",
        required=False,
        help="run the standard calibration CASA pipeline",
    )
    parser.add_argument(
        "-i",
        "--standard-img",
        nargs=2,
        action="store",
        required=False,
        help="run the standard imaging CASA pipeline",
    )
    parser.add_argument(
        "--restore",
        required=False,
        action="store_true",
        help="run the restore measurement set CASA pipeline, use in conjunction with '-c'",
    )
    parser.add_argument(
        "--integrated",
        required=False,
        action="store_true",
        help="run an integrated calibration-imaging pipeline, use in conjunction with '-i'",
    )
    return parser


def main():
    args = arg_parser().parse_args()

    path = os.getcwd()

    if args.standard_cal is not None:
        parameters = _get_settings(pathlib.Path(path), args.standard_cal)
        if args.restore:
            parameters["product_type"] = "restore"
        else:
            parameters["product_type"] = "standard-cal"

        CalibrationLauncher(parameters).launch_casa()
        # make sure we return to the parent directory after processing
        os.chdir(path)

    elif args.standard_img is not None:
        parameters = _get_settings(pathlib.Path(path), args.standard_img)
        if args.integrated:
            parameters["product_type"] = "integrated"
        else:
            parameters["product_type"] = "standard-img"

        ImagingLauncher(parameters).launch_casa()
        # return to parent directory after processing
        os.chdir(path)
