import logging
import subprocess
import sys

from ingest_envoy.ingestion_manifest import IngestionManifest
from ingest_envoy.interfaces import IngestLauncherIF


class IngestCalibrationLauncher(IngestLauncherIF):
    def __init__(self, parameters: dict):
        self.logger = logging.getLogger("ingest_envoy")
        self.sci_product_type = "calibration"
        self.parameters = parameters
        self.staging_source_dir = (
            self.parameters["staging_area"] + "/" + self.parameters["workflowDir"]
        )

    def launch_ingestion(self) -> int:
        """
        Prepare and run ingestion script

        :return: Return code of ingestion script process
        """
        self.prepare_for_ingest()
        return self.run_ingest()

    def run_ingest(self) -> int:
        """
        Run ingestion script as a subprocess

        :return: Return code of ingestion script process
        """
        ingest_process = subprocess.run(
            ["./ingest", "--json", self.staging_source_dir],
            stdout=sys.stdout,
            stderr=sys.stderr,
        )
        return ingest_process.returncode

    def prepare_for_ingest(self):
        # 1. run collection script to create calibration tarfile
        self.run_collection_script()

        # 2. create ingestion manifest
        self.create_manifest()

    def run_collection_script(self):
        workflow_dir = self.parameters["workflowDir"]
        sdm_id = self.parameters["sdmId"]
        cal_processing_datetime = self.parameters["processingStart"]
        # run script
        subprocess.run(
            ["./calibration-table-collector.sh", workflow_dir, sdm_id, cal_processing_datetime],
            stdout=sys.stdout,
            stderr=sys.stderr,
        )

    def create_manifest(self):
        spl = self.parameters["spl"]
        telescope = self.parameters["telescope"]

        IngestionManifest(self.staging_source_dir, self.sci_product_type, spl, telescope).create()
