"""
Tests for ingest_envoy.ingest
"""
import argparse
from unittest.mock import MagicMock, patch

import ingest_envoy.ingest as ingest

expected_settings = {
    "sdmId": "16B-069_sb32814386_1_001.57685.66193635417",
    "workflowName": "std_calibration",
    "workflowRequestId": "2",
    "spl": "uid://evla/execblock/48ba4c9d-d7c7-4a8f-9803-1115cd52459b",
    "processingStart": "2021_07_06T21_50_48",
    "destinationDir": "/lustre/aoc/cluster/pipeline/docker/workspaces/spool/tmp576dknjp",
    "workflowDir": "tmp576dknjp",
    "staging_area": "/lustre/aoc/cluster/pipeline/docker/workspaces/staging",
    "storage_area": "/lustre/aoc/cluster/pipeline/docker/workspaces/storage",
}

args = argparse.Namespace
filename = "test/input_files/test-metadata.json"


class TestIngest:
    def test_get_settings(self):
        settings = ingest._get_settings(filename)
        assert settings["sdmId"] == expected_settings["sdmId"]
        assert settings["workflowName"] == expected_settings["workflowName"]
        assert settings["workflowRequestId"] == expected_settings["workflowRequestId"]
        assert settings["spl"] == expected_settings["spl"]
        assert settings["processingStart"] == expected_settings["processingStart"]
        assert settings["destinationDir"] == expected_settings["destinationDir"]
        assert settings["workflowDir"] == expected_settings["workflowDir"]
        assert settings["staging_area"] == expected_settings["staging_area"]
        assert settings["storage_area"] == expected_settings["storage_area"]

    def test_check_ingest_type(self):
        arg_type = "calibration"
        arg_type2 = "image"

        check = ingest.check_ingest_type(arg_type, expected_settings)
        assert check is True

        check2 = ingest.check_ingest_type(arg_type2, expected_settings)
        assert check2 is False

    def test_send_ingestion_complete_message(self):
        fake_workflow_request_id = "2"
        expected_msg = {
            "type": "ingestion-complete",
            "workflow_request_id": fake_workflow_request_id,
        }
        with patch("ingest_envoy.ingest.Router") as patched_router:
            ingest.send_ingestion_complete_message(fake_workflow_request_id)
            patched_router.return_value.send_message.assert_called_with(**expected_msg)

    def test_parser(self):
        parser = ingest.arg_parser()
        assert isinstance(parser, argparse.ArgumentParser)

    def test_main_calibration(self):
        args.calibration = [filename]

        with patch("argparse.ArgumentParser.parse_args", MagicMock(return_value=args)) as mock_args:
            with patch(
                "ingest_envoy.launchers.IngestCalibrationLauncher.launch_ingestion", return_value=0
            ) as cal_ingest:
                with patch("ingest_envoy.ingest.send_ingestion_complete_message") as patched_send:
                    ingest.main()
                    assert cal_ingest.call_count == 1
                    assert cal_ingest.return_value == 0
                    patched_send.assert_called_with("2")

        # reset for other testing
        args.calibration = None
